@@ -49,6 +49,8 @@
 #'
 #' @import dplyr
 #' @importFrom stats ppois
+#' @importFrom reshape2 melt
+#' @importFrom reshape2 dcast
 #'
 #' @examples
 #'
@@ -151,7 +153,7 @@ denovolyze <- function(genes,classes,nsamples,
                                    "lof","misD")] <- "protD"
   input$class.5 <- "all"
 
-  input <- reshape::melt.data.frame(input,id.vars="gene") %>%
+  input <- reshape2::melt(input,id.vars="gene") %>%
     dplyr::select(gene, class = value)  %>%
     filter(!is.na(class)) %>%
     filter(class!="mis_notFilter")
@@ -166,11 +168,6 @@ denovolyze <- function(genes,classes,nsamples,
       summarise(
         observed = n()
       )
-    observed$class <- factor(observed$class, levels=c(c("syn","misD","mis",
-                                                        "non","stoploss","startloss",
-                                                        "splice","frameshift","lof","prot","protD","all")))
-    observed <- observed[order(observed$class),]
-
 
     expected <- probTable %>%
       filter(gene %in% includeGenes, class %in% includeClasses) %>%
@@ -178,11 +175,7 @@ denovolyze <- function(genes,classes,nsamples,
       summarise(
         expected = 2*sum(value, na.rm=T)*nsamples
       )
-    expected$class <- factor(expected$class, levels=c(c("syn","misD","mis",
-                                                        "non","stoploss","startloss",
-                                                        "splice","frameshift","lof","prot","protD","all")))
-
-
+    expected$class <- as.character(expected$class)
     output <- left_join(observed,expected,by=c("class"))
 
   } else if(groupBy=="gene"){
@@ -200,11 +193,12 @@ denovolyze <- function(genes,classes,nsamples,
       summarise(
         expected = 2*sum(value, na.rm=T)*nsamples
       )
+    exp$class <- as.character(exp$class)
 
     output <- merge(obs,exp,by=c("gene","class"),all=T)
   }
 
-  # calculate poisson stats and enrichments ____________________
+    # calculate poisson stats and enrichments ____________________
   output[is.na(output)] <- 0
   output$enrichment <- signif(output$observed/output$expected,signifP)
   output$pValue <- signif(ppois(output$observed-1,lambda=output$expected,lower.tail=F),signifP)
@@ -214,43 +208,41 @@ denovolyze <- function(genes,classes,nsamples,
     output$expected <- round(output$expected,roundExpected)
   }
 
-  # when analysing by gene, apply additional formatting to arrange results for different variant classes side by side
-  #  --------------------------
-  if(groupBy=="gene"){
-
-    #For single gene, format 1 row per class.  Else 1 row per gene.
-    if(length(includeGenes)==1){
-      output <-  dplyr::select(output,-enrichment)
-    } else {
-      output <- output %>%
-        dplyr::select(-enrichment) %>%
-        reshape::recast(id.var=c("gene","class"), formula = gene ~ variable ~ class)
-
-      classNotRepresented <- includeClasses[!includeClasses %in% dimnames(output)$class]
-      if (length(classNotRepresented)!=0){
-        warning(paste(classNotRepresented,"is not found in data"))
-        includeClasses[includeClasses %in% unique(output$class)]
-      }
-
-      output2 <- list()
-      for (i in seq(along=includeClasses)){
-        output2[[i]] <- as.data.frame(output[,,i])
-        names(output2[[i]]) <- paste(includeClasses[i],names(output2[[i]]),sep=".")
-      }
-
-      output3 <- output2[[1]]
-      for (i in seq(along=includeClasses)[-1]){
-        output3 <- merge(output3,output2[[i]],by="row.names",all=T)
-        rownames(output3) <- output3$Row.names
-        output3 <- dplyr::select(output3,-Row.names)
-      }
-
-      myIndex <- output3 %>% dplyr::select(ends_with("pValue")) %>% apply(MARGIN=1,min, na.rm=T) %>% order()
-
-      output <- output3[myIndex,]
-    }
+  #warn if includeClasses includes variant classes not found in the data
+  extraClasses <- includeClasses[!includeClasses %in% output$class]
+  if(length(extraClasses)>0){
+    extraClasses <- paste(extraClasses,collapse=", ")
+    warning(paste("The following variant classes specified in includeClasses are not represented in the data:",extraClasses))
   }
+
+  # impose a standard order on the variant classes for uniform output
   # --------------------------
+  # start with standard list of classes in preferred order
+  standardClassLevels <- c("syn","misD","mis",
+                     "non","stoploss","startloss",
+                     "splice","frameshift","lof","prot","protD","all")
+  # append any classes in probTable that are not already in this list
+  inUseClasses <- as.character(unique(probTable$class))
+  extraClassLevels <- inUseClasses[!inUseClasses %in% standardClassLevels]
+  myClassLevels <- c(standardClassLevels,
+                     extraClassLevels)
+  # factorise classes & drop redundant levels
+  output$class <- factor(output$class, levels=myClassLevels) %>% droplevels
+  output <- output %>% arrange(class)
+
+  # when analysing by gene, apply additional formatting to arrange results for different variant classes side by side
+  # only applied if >1 gene, so that for a single gene format 1 row per class.  Otherwise 1 row per gene.)
+  #  --------------------------
+  if(groupBy=="gene" & length(includeGenes)>1){
+    output <- output %>%
+      dplyr::select(-enrichment) %>%
+      reshape2::melt(id.vars=c("gene","class")) %>%
+      reshape2::dcast(formula = gene ~ class + variable)
+    #order by p-value
+    myIndex <- output %>% dplyr::select(ends_with("pValue")) %>% apply(MARGIN=1,min, na.rm=T) %>% order()
+    output <- output[myIndex,]
+  }
+
   class(output) <- "data.frame"
   return(output)
 }
